@@ -6,6 +6,7 @@ use WebService::Solr;
 use Catmandu::Store::Solr::Bag;
 
 with 'Catmandu::Store';
+#with 'Catmandu::Transactionable';
 
 =head1 NAME
 
@@ -13,7 +14,7 @@ Catmandu::Store::Solr - A searchable store backed by Solr
 
 =cut
 
-our $VERSION = '0.0207';
+our $VERSION = '0.0208';
 
 =head1 SYNOPSIS
 
@@ -28,9 +29,15 @@ our $VERSION = '0.0207';
     # Force an id in the store
     my $obj2 = $store->bag->add({ _id => 'test123' , name => 'Nicolas' });
 
-    # Commit all changes
+    # send all changes to solr (committed automatically)
     $store->bag->commit;
 
+    #transaction: rollback issued after 'die'
+    $store->transaction(sub{
+        $bag->delete_all();
+        die("oops, didn't want to do that!");
+    });
+
     my $obj3 = $store->bag->get('test123');
 
     $store->bag->delete('test123');
@@ -61,6 +68,31 @@ sub _build_solr {
     WebService::Solr->new($_[0]->url, {autocommit => 0, default_params => {wt => 'json'}});
 }
 
+sub transaction {
+    my($self,$sub)=@_;
+
+    if($self->{_tx}){
+        return $sub->();
+    }
+    my $solr = $self->solr;
+    my @res;
+
+    eval {
+        $self->{_tx} = 1;
+        @res = $sub->();
+        $solr->commit;
+        $self->{_tx} = 0;
+        1;
+    } or do {
+        my $err = $@;
+        eval { $solr->rollback };
+        $self->{_tx} = 0;
+        die $err;
+    };
+
+    @res;
+}
+
 =head1 SUPPORT
 
 Solr schemas need to support an identifier field (C<_id> by default) and a bag
@@ -84,6 +116,31 @@ Field that C<_bag> is mapped to in Solr
 
 =back
 
+=head1 METHODS
+
+=head2 transaction
+
+When you issue $bag->commit, all changes made in the buffer are sent to solr, along with a commit.
+So committing in Catmandu merely means flushing changes;-).
+
+When you wrap your subroutine within 'transaction', this behaviour is disabled temporarily.
+You still need to call 'commit', but this only sents the documents to Solr without commit.
+When you call 'die' within the subroutine, a rollback is sent to solr.
+
+#record 'test' added
+$bag->add({ _id => "test" });
+#buffer flushed, and 'commit' sent to solr
+$bag->commit();
+
+$bag->store->transaction(sub{
+    $bag->add({ _id => "test",title => "test" });
+    $bag->commit;
+    #call to die: rollback sent to solr
+    die("oops, didn't want to do that!");
+});
+
+#record is still { _id => "test" }
+
 =head1 SEE ALSO
 
 L<Catmandu::Store>, L<WebService::Solr>
